'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.renderHead = renderHead;
exports.renderFooter = renderFooter;
exports.default = RenderApp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _Middleware = require('./Middleware');

var _Middleware2 = _interopRequireDefault(_Middleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function renderHead(head, additionalStylesheets = []) {
  const metaTags = head.meta || [];
  const links = head.links || [];
  const scripts = head.scripts || [];
  const stylesheets = [...(head.stylesheets || []), ...additionalStylesheets];

  return (0, _server.renderToStaticMarkup)(_react2.default.createElement(
    'head',
    null,
    head.base && _react2.default.createElement('base', head.base),
    metaTags.map((tag, i) => _react2.default.createElement('meta', _extends({ key: i }, tag))),
    _react2.default.createElement(
      'title',
      null,
      head.title
    ),
    links.map((linkAttrs, i) => _react2.default.createElement('link', _extends({ key: i }, linkAttrs))),
    stylesheets.map((stylesheetAttrs, i) => _react2.default.createElement('link', _extends({ key: `s-${i}` }, stylesheetAttrs, { rel: stylesheetAttrs.rel || 'stylesheet' }))),
    scripts.map((scriptAttrs, i) => _react2.default.createElement('script', _extends({
      key: `scr-${i}`
    }, scriptAttrs, {
      dangerouslySetInnerHTML: { __html: scriptAttrs.script }
    })))
  ));
}

function renderFooter(scripts = []) {
  return (0, _server.renderToStaticMarkup)(_react2.default.createElement(
    'footer',
    null,
    scripts.map((scriptAttrs, i) => _react2.default.createElement('script', _extends({
      key: `fscr-${i}`
    }, scriptAttrs, {
      dangerouslySetInnerHTML: { __html: scriptAttrs.script }
    })))
  )).replace(/<(\/)?footer>/g, '');
}

function RenderApp({ render }) {
  return _react2.default.createElement(_Middleware2.default, {
    use: (() => {
      var _ref = _asyncToGenerator(function* (ctx) {
        const {
          body = '',
          containerId = 'app',
          lang = 'en',
          head = {},
          scripts = [],
          status,
          stylesheets = []
        } = yield render(ctx);

        ctx.status = status || 200;

        ctx.body = `
<html lang="${lang}">
  ${renderHead(head, stylesheets)}
  <body>
    <div id="${containerId}">${body}</div>
    ${renderFooter(scripts)}
  </body>
</html>
    `.trim();
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })()
  });
}